#!/usr/bin/env python

import os
import re
import sys
from fnmatch import fnmatch
from datetime import datetime

files = os.listdir('.')
files.sort()

octet = r'(?:2(?:[0-4]\d|5[0-5])|1\d\d|\d{1,2})'                      
ipAddr = r'%s(?:\.%s){3}' % (octet, octet)                            
label = r'[\w][-\w]*[\w]?'                                
domain = r'%s(?:\.%s)*\.[a-z][-\w]*[a-z]?' % (label, label)       
url = r'(\w+://(?:%s|%s)(?::\d+)?(?:/[^\])>\s]*)?)' % (domain, ipAddr)

urlre = re.compile(url, re.I + re.U)

channel = sys.argv[1]
ignorelist = sys.argv[2:]
ignorelist.append('***')

from storm.locals import *
from storm.sqlobject import AutoUnicode

urls_schema = \
"""CREATE TABLE urls (\
    id INTEGER PRIMARY KEY AUTOINCREMENT,\
    channel TEXT COLLATE NOCASE,\
    nick TEXT COLLATE NOCASE,\
    url TEXT COLLATE NOCASE,\
    date DATETIME,
    UNIQUE (channel, url))
"""

others_schema = \
"""CREATE TABLE other_urls (\
    id INTEGER PRIMARY KEY AUTOINCREMENT,\
    channel TEXT COLLATE NOCASE,\
    nick TEXT COLLATE NOCASE,\
    url_id INTEGER references urls(id),\
    date DATETIME)
"""


class Url(object):
    __storm_table__ = 'urls'

    id = Int(primary=True)
    channel = AutoUnicode()
    nick = AutoUnicode()
    url = AutoUnicode()
    date = DateTime()

    def __init__(self, url, nick, channel, date):
        self.url = url
        self.nick = nick
        self.channel = channel
        self.date = date


class OtherUrl(object):
    __storm_table__ = 'other_urls'

    id = Int(primary=True)
    channel = AutoUnicode()
    nick = AutoUnicode()
    url_id = Int()
    date = DateTime()

    url = Reference(url_id, Url.id)

    def __init__(self, nick, channel, date):
        self.nick = nick
        self.channel = channel
        self.date = date


db = create_database('sqlite:Urldb.sqlite.db')
store = Store(db)
try:
    store.execute(urls_schema)
    store.execute(others_schema)
    store.commit()
except:
    pass


_nicks = []
for file in files:
    if not fnmatch(file, '*.log'):
        continue
    print file
    for line in open(file):
        line = line.strip()
        try:
            line = unicode(line, 'utf-8')
        except:
            continue
        match = urlre.findall(line)
        if match:
            try:
                date, nick, _ = line.split(None, 2)
                if nick[0] == '-' and nick[-1] == '-':
                    # notice
                    continue
                if nick == '*':
                    nick = line.split()[2]
                nick = nick.strip('<>')
                if nick.lower() in ignorelist:
                    continue
                if nick not in _nicks:
                    print '  ' + nick
                    _nicks.append(nick)
                date = datetime.strptime(date, '%Y-%m-%dT%H:%M:%S')
                for url in match:
                    _url = store.find(Url, Url.url.like(url)).one()
                    if _url:
                        other = OtherUrl(nick, channel, date)
                        other.url = _url
                        store.add(other)
                    else:
                        url = Url(url, nick, channel, date)
                        store.add(url)

                store.commit()
            except:
                print line
                raise
